import 'package:rxdart/rxdart.dart';

class WidgetBloc {
  /// Contains the information related to the [key] and [value] of every [parser] class.
  final _mapper = <String, dynamic>{};

  final _widgetController = BehaviorSubject<Map<String, dynamic>>();

  /// Returns a [Stream] of [Map<String, dynamic].
  /// ```dart
  /// Map<String, dynamic> = {"textfield": "valueInTextField", "datetime": "2020/08/22"}
  Stream<Map<String, dynamic>> get widgetsStream => _widgetController.stream;

  /// Check if the [newMap] param is already registed inside the [_mapper]
  ///
  /// and return a [Map<String, dynamic>].
  void registerMap(Map<String, dynamic> newMap) {
    newMap.forEach((key, value) {
      if (_mapper.containsKey(key)) {
        _mapper.update(key, (_) => value);
      } else {
        _mapper[key] = value;
      }
    });

    _widgetController.sink.add(_mapper);
  }

  /// Close the Stream.
  void dispose() {
    _widgetController?.close();
  }
}
