import 'package:flutter/material.dart';
import 'package:flutter_neumorphic/flutter_neumorphic.dart';

import 'package:formio_flutter/formio_flutter.dart';
import 'package:formio_flutter/src/abstraction/abstraction.dart';
import 'package:formio_flutter/src/models/models.dart';
import 'package:formio_flutter/src/providers/providers.dart';

/// Extends the abstract class [WidgetParser]
class ButtonParser extends WidgetParser {
  /// Returns a [Widget] of type [Button]
  @override
  Widget parse(
    Component map,
    BuildContext context,
    ClickListener listener,
    WidgetProvider widgetProvider,
  ) {
    var isVisible = true;
    var clickEvent = map.action ?? '';

    /// Declared [WidgetProvider] to consume the [Map<String, dynamic>] created from it.
    var button = (map.hidden)
        ? SizedBox.shrink()
        : StreamBuilder(
            stream: widgetProvider.widgetBloc.widgetsStream,
            builder: (context, AsyncSnapshot<Map<String, dynamic>> snapshot) {
              isVisible = (map.conditional != null && snapshot.data != null)
                  ? (snapshot.data.containsKey(map.conditional.when) &&
                          snapshot.data[map.conditional.when].toString() ==
                              map.conditional.eq)
                      ? map.conditional.show
                      : !map.conditional.show
                  : true;
              return (!isVisible)
                  ? SizedBox.shrink()
                  : Padding(
                      padding:
                          EdgeInsets.symmetric(horizontal: 5.0, vertical: 5.0),
                      child: NeumorphicButton(
                        style: NeumorphicStyle(
                          color: parseHexColor(map.theme ?? "primary"),
                          shadowLightColor:
                              parseHexColor(map.theme ?? "primary"),
                        ),
                        child: (map.leftIcon != null || map.rightIcon != null)
                            ? SizedBox(
                                width: MediaQuery.of(context).size.width * 0.5,
                                child: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  children: [
                                    LeftIconWidgetParser().parse(
                                      map,
                                      context,
                                      listener,
                                      widgetProvider,
                                    ),
                                    Text(
                                      map.label,
                                      softWrap: true,
                                      style: TextStyle(
                                        fontSize: 20.0,
                                        color: Colors.white,
                                      ),
                                    ),
                                    RightIconWidgetParser().parse(
                                      map,
                                      context,
                                      listener,
                                      widgetProvider,
                                    ),
                                  ],
                                ),
                              )
                            : Text(
                                map.label,
                                softWrap: true,
                                style: TextStyle(
                                  fontSize: 20.0,
                                  color: Colors.white,
                                ),
                              ),
                        onPressed: (map.disabled)
                            ? null
                            : () => listener.onClicked(clickEvent),
                      ),
                    );
            },
          );
    return button;
  }

  /// [widgetName] => "button"
  @override
  String get widgetName => "button";
}
